#!/usr/bin/env python
# -*- coding: utf-8 -*-


repos = ['-TWO-EARS-_navigation', '3rd-party-ressources', 'ae-scoot', 'AGVC', 'AGV_NUC', 'air1', 'alpha_autopilot', 'andbot_simulator', 'andbot_tool', 'arcas_ws', 'ardrone_follower', 'ardrone_swarm', 'asctec_mav_motion_planning', 'ASL_MULTIMAPPING_WORKSTATION', 'ati_force_torque', 'auckbot', 'augmented_odom', 'Autonomous-Vision-Based-Docking-Rob-work-3', 'babs', 'base_local_planner', 'BasketBotGame', 'bayesian_compact_ware', 'bebop2_controller', 'bk_nav', 'brics_research_camp', 'CIS700_Squirtle', 'CNV', 'COMP3431-Assignment1', 'contact_control', 'cs225b', 'de-floribot-software', 'dextrus_ws', 'docker-quad-moveit', 'dynamic_mapping', 'EECS476_final_project', 'ek-ssl', 'eklavya-2015', 'endoscopy_capsule-project', 'epochmini', 'ewyn2', 'four_wd_rover', 'FRASIER_Robot_WPI', 'FrenchVanilla', 'frobo_rsd', 'gazebo_ros_wheel', 'gcs_agent', 'gki_3dnav', 'gki_navigation', 'GLM_AGV', 'go2point..', 'graveyard', 'hbrs-youbot-hackathon', 'hku_m100_gazebo', 'humanoid_catching', 'iarc_software', 'idmind_ros', 'IGV_Software', 'InterestPointCorrelationSupportMaterials', 'iri_navigation', 'isr-uc-ros-pkg', 'Jaguar4x4', 'jaguar_base', 'jeevesrobot', 'jsk_mbzirc_task3', 'jsk_recognition', 'jsk_visualization_packages', 'kinematic_calibration', 'laser_collision_detection', 'last_letter', 'Local_Planners', 'lucylin0821-pr2-band', 'Manipulator_teleoperation', 'marker_tracker', 'melvin', 'micROS-SARRT-GlobalPlanner', 'mobile_robots_lab_ss15', 'mobrob_robin', 'mocap_kalman', 'mrpt_slam', 'multy_rob', 'muro_lab_summer', 'Nav_Stack', 'NegarNemati_NiranjanDeshpande_TeenaHassan', 'Neo_Robot', 'netft_utils', 'nice', 'nonstop_robots', 'Notre-bon-plaisir-Pascal-Bauer', 'nucobot', 'octagon', 'odom_translation', 'oldcomputer', 'omnimaxbot', 'OnlineSCRAM', 'pacman-DR54', 'pandabot', 'parallel_uav', 'PbD_FRI', 'pff-node-ws', 'phriky-units', 'plugin_local_planner', 'pololu_imu', 'pr2_arm_base_control', 'pr2_controllers', 'pr2_doors', 'pr2_object_manipulation', 'pr2_placing', 'pthead', 'px4controller', 'px4_offboard', 'px4_offboard_velocity_control', 'quadroter_auto_nav', 'quadrotor-artag-navigation', 'race_solutions', 'ransac_catkin', 'RAS2016', 'RBE_510_ROS', 'REPO', 'research-camp-5', 'rhocode', 'rlfd_ast3', 'rmc_software', 'robocup-at-work', 'Robot', 'Robot-Algs', 'Robots-OD', 'robot_movement_interface', 'robo_ctrl', 'ROS-CATEC', 'ros-hydro', 'ROS-Hydro-SLAM', 'ROS-labbot', 'ros-navigation', 'ros-vrep', 'ros_common_robin', 'ROS_Hand_Tracker', 'ros_phidgets_jade', 'ROS_pidcalculator', 'ros_python3', 'RO_1516', 'RTB', 'ru_pracsys', 'SDI-11911', 'segbot_arm', 'Senior-Project', 'SImu_robot_holonome', 'smartlabatwork-release', 'soma_ur5', 'SpatioTemporalExploration', 'speed_control', 'spiri_navigation-1', 'SpringerROS_Gazebo2015', 'srrc_img2dir', 'sshbot', 'stanley', 'subcultron-sim', 'suturo_manipulation', 'swarmlabatwork', 'switch_vis_exp', 'task_motion_planning_cds', 'teach-a-robot', 'TinyRobo', 'Toretto', 'trik-ros', 'TRIN_EARL', 'tukuba_challenge_2014', 'turtlebot', 'tut_ibx0020', 'uavros', 'uav_control_ros', 'uav_object_localisation', 'upo_robot_navigation', 'usma_optitrack', 'uxvcos', 'viconxbee', 'VilmaProject', 'vision_ardrone', 'visoRos', 'visp_cam_vs', 'wasp_search_and_rescue', 'wpi-dvrk-ros', 'wrench_filter', 'xm_velocity_smoother', 'youbot_controllers', 'yujin_ocs', 'zeta_open_loop', 'zsROS']


import random
from datetime import datetime
random.seed(datetime.now())

print repos[random.randint(0,len(repos)-1)]
